#include <avr/io.h>
#include <avr/interrupt.h>

#include "config.h"
#include "timer.h"
#include "energy.h"
#include "vbus.h"
#include "nrg_meter.h"
#include "stack.h"
#include "buff.h"
#include "ntp.h"
#include "timer.h"
#include <string.h>

const char progmem_true[] PROGMEM = "true";
const char progmem_false[] PROGMEM = "false";

volatile uint16_t nrg_timer = 900;
volatile uint8_t nrg_ticks;
volatile uint8_t nrg_ticksb;

volatile uint8_t nrg_tick;
volatile uint8_t nrg_arrpos;


volatile t_vbus_outdata c_solar;
volatile t_vbus_outdata c_solar_tmp;

volatile uint16_t c_air_pressure;
volatile uint8_t  c_wp_ht;
volatile uint8_t  c_wp_pwr;
volatile uint8_t  c_solar_lowpressure = 0;

inline void nrg_timer_init() {
	TCCR0A = 0; 
	TCCR0B = (1<<CS02) | (0<<CS01) | (1<<CS00); //Prescaler 1024 -> 15625Hz

	TIMSK0 |= 1<<TOIE0; //enable overflow INT 
}

//~61Hz
ISR (TIMER0_OVF_vect) {
	nrg_tick++;
	if(nrg_tick == 2) {
		//~30Hz
		
		nrgmtr_check((t_nrgmtr *)&nrg_meters[0]); //Hauptzhler
		if(energy_stats[nrg_arrpos].wp_ht == 1) {
			nrgmtr_check((t_nrgmtr *)&nrg_meters[1]); //WP HT
		} else {
			nrgmtr_check((t_nrgmtr *)&nrg_meters[2]); //WP NT
		}
		nrgmtr_check((t_nrgmtr *)&nrg_meters[3]); //Wasser
		
		
		nrg_tick = 0;
	}
	
}

void nrg_init() {
	nrg_timer_init();
	nrg_arrpos = 0;

	NRG_WP_PEAK_DDR &= ~NRG_WP_PEAK_PIN;
	NRG_WP_PWR_DDR &= ~NRG_WP_PWR_PIN;
	//NRG_SOLAR_LOWP_DDR &= ~NRG_SOLAR_LOWP_PIN

	nrgmtr_init((t_nrgmtr *)&nrg_meters[0], &DDRA, &PINA, PA6); //Hauptzhler
	nrgmtr_init((t_nrgmtr *)&nrg_meters[1], &DDRA, &PINA, PA7); //WP Haupttarif
	nrgmtr_init((t_nrgmtr *)&nrg_meters[2], &DDRA, &PINA, PA7); //WP Nebentarif
	nrgmtr_init((t_nrgmtr *)&nrg_meters[3], &DDRC, &PINC, PC2); //Wasser

	VBus_RequestData((t_vbus_outdata *)&c_solar_tmp);
}


void nrg_mainloop() {
	//Zehntelsekunden!
	if(nrg_ticks > 0) {
		//Zhler aktualisieren
		for(uint8_t i = 0; i < NRG_METER_CNT; i++) {
			nrgmtr_tick((t_nrgmtr *)&nrg_meters[i], nrg_ticks);
		}
		nrg_ticksb += nrg_ticks;
		nrg_ticks = 0;

		//nur neue Abfrage starten, wenn alte abgeschlossen wurdee
		if(vbus_head.valid == 1) {
			memcpy((char *)&c_solar, (char *)&c_solar_tmp, sizeof(t_vbus_outdata));
			VBus_RequestData((t_vbus_outdata *)&c_solar_tmp);
		}
	}

	//10 Sekunden
	if(nrg_ticksb >= 100) {
		nrg_ticksb = 0;

		c_wp_ht = (NRG_WP_PEAK_REG & NRG_WP_PEAK_PIN) ? 1 : 0;
		c_wp_pwr = (NRG_WP_PWR_REG & NRG_WP_PWR_PIN) ? 1 : 0;


		

	}

	//nrg_timer mit der echten Zeit synchronisieren, damit die Zeit immer im gleichen Zeitraster ist
	if(time % NRG_LOG_TIMESLOT == 0 && nrg_timer > 30) {
		nrg_timer = NRG_LOG_TIMESLOT;
	}

	//Viertel Stunde vorbei
	if(nrg_timer >= NRG_LOG_TIMESLOT) {
	//if(nrg_timer >= 10) {
		nrg_timer = 0;

		//Zhlerwerte fr letzte Zeiteinheit bernehmen
		for(uint8_t i = 0; i < NRG_METER_CNT; i++) {
			energy_stats[nrg_arrpos].meter[i] = nrg_meters[i].umlaeufe;
			nrg_meters[i].umlaeufe = 0; //Umlufe zurcksetzen
		}
		
		//auf Speicher fr neue Zeiteinheit springen
		nrg_arrpos++;
		if(nrg_arrpos >= NRG_LOG_COUNT) {
			nrg_arrpos = 0;
		}

		//Uhrzeit bernehmen
		energy_stats[nrg_arrpos].starttime = GetUnixTimestamp(time);
		
		//Aktuelle Daten der Solaranlage bernehmen
		memcpy((char *)&energy_stats[nrg_arrpos].solar, (char *)&c_solar, sizeof(t_vbus_outdata));
		
		//Bogus-Werte lschen
		for(uint8_t i = 0; i < NRG_METER_CNT; i++) {
			energy_stats[nrg_arrpos].meter[i] = 0;
		}

		energy_stats[nrg_arrpos].air_pressure = c_air_pressure;
		energy_stats[nrg_arrpos].wp_ht = c_wp_ht == 0 ? 0 : 1;
		energy_stats[nrg_arrpos].wp_pwr = c_wp_pwr == 0 ? 0 : 1;
		energy_stats[nrg_arrpos].solar_lowpressure = c_solar_lowpressure == 0 ? 0 : 1;
		

	}
}

uint8_t nrg_ToJson_Current(char *ptr, uint8_t index) {
	uint8_t pos = 0;
	
	if(index == 0) {
		//Zeit
		pos = buff_str_p("{\"time\":", ptr, pos, BUFFER_LEN); //{"time":
		pos = buff_int(GetUnixTimestamp(time), ptr, pos, BUFFER_LEN, 10);

		//Uptime
		pos = buff_str_p(",\"uptime\":", ptr, pos, BUFFER_LEN); //,"uptime":
		pos = buff_int(uptime, ptr, pos, BUFFER_LEN, 10);

		//Luftdruck
		pos = buff_str_p(",\"air_press\":", ptr, pos, BUFFER_LEN); //"],"air_press":
		pos = buff_int(c_air_pressure, ptr, pos, BUFFER_LEN, 10);
	
		//Wrmepumpenstatus
		pos = buff_str_p(",\"wp\":{\"ht\":", ptr, pos, BUFFER_LEN); //","wp":{"ht":
		pos = buff_str_fla(c_wp_ht?progmem_true:progmem_false, ptr, pos, BUFFER_LEN);
		pos = buff_str_p(",\"pwr\":", ptr, pos, BUFFER_LEN); //,"pwr":
		pos = buff_str_fla(c_wp_pwr?progmem_true:progmem_false, ptr, pos, BUFFER_LEN);

		pos = buff_str_p("},\"solar\":{\"low_p\":", ptr, pos, BUFFER_LEN); //},"solar":{"low_p":
		pos = buff_str_fla(c_solar_lowpressure?progmem_true:progmem_false, ptr, pos, BUFFER_LEN);

		pos = buff_str_p(",\0", ptr, pos, BUFFER_LEN);

		return 1;
	} 
	if(index > 0 && index <= JSON_VBUS_REPORTS) {
		VBus_ToJson((t_vbus_outdata *)&c_solar, ptr, index - 1);
		return 1;
	}
	if(index == (JSON_VBUS_REPORTS + 1)) {
		pos = buff_str_p("},\"meters\":[{", ptr, pos, BUFFER_LEN); //},"meters":[{
		return 1;
	}
	if(index >= (JSON_VBUS_REPORTS + 2) && index < (JSON_VBUS_REPORTS + JSON_NRGMTR_REPORTS * NRG_METER_CNT + 2)) {
		pos = nrgmtr_ToJson((t_nrgmtr *)&nrg_meters[index - (JSON_VBUS_REPORTS + 2)], ptr);

		if(index == JSON_VBUS_REPORTS + JSON_NRGMTR_REPORTS * NRG_METER_CNT + 1) {
			pos = buff_str_p("}]", ptr, pos, BUFFER_LEN); //}]
		} else {
			pos = buff_str_p("},{", ptr, pos, BUFFER_LEN); //},{
		}
		return 1;
	}
	if(index == (JSON_VBUS_REPORTS + JSON_NRGMTR_REPORTS * NRG_METER_CNT + 2)) {
		pos = buff_str_p("}\0", ptr, pos, BUFFER_LEN); //}
		return 1;
	}


	return 0;
}

uint8_t nrg_ToJson_History(char *ptr, uint8_t index) {
	

	uint8_t pos = 0;


	if(index == 0) {
		pos = buff_str_p("[\0", ptr, pos, BUFFER_LEN);
		return 1;
	}

	if(index == NRG_LOG_COUNT * (JSON_VBUS_REPORTS+3)) {
		pos = buff_str_p("]\0", ptr, pos, BUFFER_LEN); //,
		 return 1; 
	}
	if(index == NRG_LOG_COUNT * (JSON_VBUS_REPORTS+3) + 1) return 0;

	index -= 1; //index des nchsten Pakets
	uint8_t offset = index/(JSON_VBUS_REPORTS+3);
	index = index % (JSON_VBUS_REPORTS+3);
	

	if(index == JSON_VBUS_REPORTS+2) {
		pos = buff_str_p(",", ptr, pos, BUFFER_LEN); //,
		//pos = buff_str_p("next stream follows\r\n\0", ptr, pos, BUFFER_LEN);
		return 1;

	}
	
	t_nrg_stats * stats = (t_nrg_stats *)&energy_stats[offset];
	

	if(index == 0) {
		pos = buff_str_p("{\"starttime\":", ptr, pos, BUFFER_LEN); //{"starttime":
	
		//Zeit
		pos = buff_int(stats->starttime, ptr, pos, BUFFER_LEN, 10);

		//Luftdruck
		pos = buff_str_p(",\"air_press\":", ptr, pos, BUFFER_LEN); //"],"air_press":
		pos = buff_int(stats->air_pressure, ptr, pos, BUFFER_LEN, 10);
	
		//Wrmepumpenstatus
		pos = buff_str_p(",\"wp\":{\"ht\":", ptr, pos, BUFFER_LEN); //","wp":{"ht":
		pos = buff_str_fla(stats->wp_ht?progmem_true:progmem_false, ptr, pos, BUFFER_LEN);
		pos = buff_str_p(",\"pwr\":", ptr, pos, BUFFER_LEN); //,"pwr":
		pos = buff_str_fla(stats->wp_pwr?progmem_true:progmem_false, ptr, pos, BUFFER_LEN);
		
		//Solaranlage
		pos = buff_str_p("},\"solar\":{\"low_p\":", ptr, pos, BUFFER_LEN); //},"solar":{"low_p":
		pos = buff_str_fla(stats->solar_lowpressure?progmem_true:progmem_false, ptr, pos, BUFFER_LEN);

		pos = buff_str_p(",\0", ptr, pos, BUFFER_LEN);

		return 1;
	} 
	if(index > 0 && index <= JSON_VBUS_REPORTS) {
		VBus_ToJson((t_vbus_outdata *)&stats->solar, ptr, index - 1);
		return 1;
	}
	if(index == (JSON_VBUS_REPORTS + 1)) {
		pos = buff_str_p("},\"meters\":[", ptr, pos, BUFFER_LEN); //},"meters":[
		for(uint8_t i = 0; i < 4; i++) {
			pos = buff_int(stats->meter[i], ptr, pos, BUFFER_LEN, 10);
			pos = buff_str_p(",", ptr, pos, BUFFER_LEN); //,
		}
		pos--;
		pos = buff_str_p("]}", ptr, pos, BUFFER_LEN);
			
		
		return 1;
	}


	return 0;
}

volatile uint8_t nrg_reportindex = 0;
volatile uint8_t nrg_buffoffset = 0;


void nrg_put_http(uint8_t type, uint8_t tcp_entryid, uint8_t packetindex) {
//	uint8_t needdata = 0;	

	uint8_t (*foo)(char *ptr, uint8_t index);
	if(type == 1) {
		foo =(void *)&nrg_ToJson_Current;
	} else {
		foo = (void *)&nrg_ToJson_History;
	}

	//erstes Paket, Zhler zurcksetzen und erstes Paket holen
	if(packetindex == 0) {
		
		nrg_buffoffset = 0;
		//ersten Pufferinhalt einlesen
		foo(buff, 0);
		nrg_reportindex = 1;
	}

	uint16_t eth_pos = TCP_DATA_START;
	
	while(1) {
		
			//Datenmenge wurde berschritten -> Paket senden
			if(eth_pos >= (MTU_SIZE-(TCP_DATA_START)-10)) {
				tcp_entry[tcp_entryid].status = ACK_FLAG | PSH_FLAG;
				create_new_tcp_packet(eth_pos - TCP_DATA_START, tcp_entryid);
				return;
			}

			//nullzeichen erkannt -> neue Daten anfordern!
			if(buff[nrg_buffoffset] == 0 || nrg_buffoffset >= BUFFER_LEN) {
				nrg_buffoffset = 0;
				if(foo(buff, nrg_reportindex++) == 0) {
					//kommen keine Daten mehr
					tcp_entry[tcp_entryid].app_status = 0xFFFD;
					tcp_entry[tcp_entryid].status = ACK_FLAG | PSH_FLAG;
					create_new_tcp_packet(eth_pos - TCP_DATA_START, tcp_entryid);
					return;
				}
				//break;
			}
			eth_buffer[eth_pos] = buff[nrg_buffoffset];
			eth_pos++;
			nrg_buffoffset++;
		
	}
}















