#include "nrg_meter.h"
#include "buff.h"

void nrgmtr_init(t_nrgmtr* z, volatile unsigned char *in_ddr, volatile unsigned char *in_reg, uint8_t in_pin) {
	*in_ddr &= ~(1<<in_pin);
	z->in_reg = in_reg;
	z->in_pin = (1<<in_pin);
	z->umlaeufe = 0;
	z->letzt_umlauf = 0xFFFF;
}

inline uint8_t nrgmtr_check(t_nrgmtr* z) {
	z->mask <<=1;
	if(*z->in_reg & z->in_pin) {
		z->mask |= 1;
	}

	if(z->mask == 0xF0) {
		z->letzt_umlauf = z->umlaufzeit;
		z->umlaufzeit = 0;
		z->umlaeufe++;
		return 1;
	}
	return 0;
}

inline void nrgmtr_tick(t_nrgmtr* z, uint8_t time) {
	//berlauf vermeiden, da es sonst zu extrem hohen Werten kommt
	if(z->umlaufzeit < 0xFFFF) {
		z->umlaufzeit += time;
	}
}

inline uint16_t nrgmtr_verbrauch(uint16_t umlauf, uint8_t timerfreq, uint8_t rotperkwh) {
	return ((1000 * timerfreq) / umlauf) * (3600 / rotperkwh);
}

/*
inline uint16_t zaehler_verbrauch(uint16_t zeit) {
	uint16_t dilution = 10;
	uint16_t corra = ((0xFFFF / 3600 / dilution) * ROT_PER_KWH) / TIMER_FREQUENCY;
	uint16_t corrb = zeit / (1000 / dilution);
	uint16_t corr = corra * corrb;

	//lcd_print(0,0, "%5i %5i %4i", corra, corrb, corr);

	if(corr == 0) corr = 1; //div/0 vermeiden

	uint16_t corrzeit = zeit / corr;
	
	uint16_t value = ((1000 * TIMER_FREQUENCY) / corrzeit) * (3600 / ROT_PER_KWH);
	//lcd_print(1,0, "%5i %5i", corrzeit, value);
	return value / corr;
}
*/

uint8_t nrgmtr_ToJson(t_nrgmtr *data, char *ptr) {
	uint8_t pos = 0;

	pos = buff_str_p("\"uml\":", ptr, pos, BUFFER_LEN); //{"uml":
	pos = buff_int(data->umlaeufe, ptr, pos, BUFFER_LEN, 10);

	pos = buff_str_p(",\"t_uml\":", ptr, pos, BUFFER_LEN); //{"t_uml":
	pos = buff_int(data->umlaufzeit, ptr, pos, BUFFER_LEN, 10);

	pos = buff_str_p(",\"t_uml_l\":", ptr, pos, BUFFER_LEN); //,"t_uml_l":
	pos = buff_int(data->letzt_umlauf, ptr, pos, BUFFER_LEN, 10);

	pos = buff_str_p("\0", ptr, pos, BUFFER_LEN);
	return pos;
}
